package jdk.internal.org.objectweb.asm.commons;

import jdk.internal.org.objectweb.asm.Label;
import jdk.internal.org.objectweb.asm.Handle;
import jdk.internal.org.objectweb.asm.Type;
import jdk.internal.org.objectweb.asm.MethodVisitor;

public class InstructionAdapter extends MethodVisitor
{
    public static final Type OBJECT_TYPE;
    
    public InstructionAdapter(MethodVisitor methodVisitor) {
        this(327680, methodVisitor);
        if (this.getClass() != InstructionAdapter.class) {
            throw new IllegalStateException();
        }
        return;
    }
    
    protected InstructionAdapter(int api, MethodVisitor methodVisitor) {
        super(api, methodVisitor);
    }
    
    @Override
    public void visitInsn(int n) {
        switch (n) {
            case 0: {
                this.nop();
                break;
            }
            case 1: {
                this.aconst(null);
                break;
            }
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
            case 7:
            case 8: {
                this.iconst(n - 3);
                break;
            }
            case 9:
            case 10: {
                this.lconst((long)(n - 9));
                break;
            }
            case 11:
            case 12:
            case 13: {
                this.fconst((float)(n - 11));
                break;
            }
            case 14:
            case 15: {
                this.dconst((double)(n - 14));
                break;
            }
            case 46: {
                this.aload(Type.INT_TYPE);
                break;
            }
            case 47: {
                this.aload(Type.LONG_TYPE);
                break;
            }
            case 48: {
                this.aload(Type.FLOAT_TYPE);
                break;
            }
            case 49: {
                this.aload(Type.DOUBLE_TYPE);
                break;
            }
            case 50: {
                this.aload(InstructionAdapter.OBJECT_TYPE);
                break;
            }
            case 51: {
                this.aload(Type.BYTE_TYPE);
                break;
            }
            case 52: {
                this.aload(Type.CHAR_TYPE);
                break;
            }
            case 53: {
                this.aload(Type.SHORT_TYPE);
                break;
            }
            case 79: {
                this.astore(Type.INT_TYPE);
                break;
            }
            case 80: {
                this.astore(Type.LONG_TYPE);
                break;
            }
            case 81: {
                this.astore(Type.FLOAT_TYPE);
                break;
            }
            case 82: {
                this.astore(Type.DOUBLE_TYPE);
                break;
            }
            case 83: {
                this.astore(InstructionAdapter.OBJECT_TYPE);
                break;
            }
            case 84: {
                this.astore(Type.BYTE_TYPE);
                break;
            }
            case 85: {
                this.astore(Type.CHAR_TYPE);
                break;
            }
            case 86: {
                this.astore(Type.SHORT_TYPE);
                break;
            }
            case 87: {
                this.pop();
                break;
            }
            case 88: {
                this.pop2();
                break;
            }
            case 89: {
                this.dup();
                break;
            }
            case 90: {
                this.dupX1();
                break;
            }
            case 91: {
                this.dupX2();
                break;
            }
            case 92: {
                this.dup2();
                break;
            }
            case 93: {
                this.dup2X1();
                break;
            }
            case 94: {
                this.dup2X2();
                break;
            }
            case 95: {
                this.swap();
                break;
            }
            case 96: {
                this.add(Type.INT_TYPE);
                break;
            }
            case 97: {
                this.add(Type.LONG_TYPE);
                break;
            }
            case 98: {
                this.add(Type.FLOAT_TYPE);
                break;
            }
            case 99: {
                this.add(Type.DOUBLE_TYPE);
                break;
            }
            case 100: {
                this.sub(Type.INT_TYPE);
                break;
            }
            case 101: {
                this.sub(Type.LONG_TYPE);
                break;
            }
            case 102: {
                this.sub(Type.FLOAT_TYPE);
                break;
            }
            case 103: {
                this.sub(Type.DOUBLE_TYPE);
                break;
            }
            case 104: {
                this.mul(Type.INT_TYPE);
                break;
            }
            case 105: {
                this.mul(Type.LONG_TYPE);
                break;
            }
            case 106: {
                this.mul(Type.FLOAT_TYPE);
                break;
            }
            case 107: {
                this.mul(Type.DOUBLE_TYPE);
                break;
            }
            case 108: {
                this.div(Type.INT_TYPE);
                break;
            }
            case 109: {
                this.div(Type.LONG_TYPE);
                break;
            }
            case 110: {
                this.div(Type.FLOAT_TYPE);
                break;
            }
            case 111: {
                this.div(Type.DOUBLE_TYPE);
                break;
            }
            case 112: {
                this.rem(Type.INT_TYPE);
                break;
            }
            case 113: {
                this.rem(Type.LONG_TYPE);
                break;
            }
            case 114: {
                this.rem(Type.FLOAT_TYPE);
                break;
            }
            case 115: {
                this.rem(Type.DOUBLE_TYPE);
                break;
            }
            case 116: {
                this.neg(Type.INT_TYPE);
                break;
            }
            case 117: {
                this.neg(Type.LONG_TYPE);
                break;
            }
            case 118: {
                this.neg(Type.FLOAT_TYPE);
                break;
            }
            case 119: {
                this.neg(Type.DOUBLE_TYPE);
                break;
            }
            case 120: {
                this.shl(Type.INT_TYPE);
                break;
            }
            case 121: {
                this.shl(Type.LONG_TYPE);
                break;
            }
            case 122: {
                this.shr(Type.INT_TYPE);
                break;
            }
            case 123: {
                this.shr(Type.LONG_TYPE);
                break;
            }
            case 124: {
                this.ushr(Type.INT_TYPE);
                break;
            }
            case 125: {
                this.ushr(Type.LONG_TYPE);
                break;
            }
            case 126: {
                this.and(Type.INT_TYPE);
                break;
            }
            case 127: {
                this.and(Type.LONG_TYPE);
                break;
            }
            case 128: {
                this.or(Type.INT_TYPE);
                break;
            }
            case 129: {
                this.or(Type.LONG_TYPE);
                break;
            }
            case 130: {
                this.xor(Type.INT_TYPE);
                break;
            }
            case 131: {
                this.xor(Type.LONG_TYPE);
                break;
            }
            case 133: {
                this.cast(Type.INT_TYPE, Type.LONG_TYPE);
                break;
            }
            case 134: {
                this.cast(Type.INT_TYPE, Type.FLOAT_TYPE);
                break;
            }
            case 135: {
                this.cast(Type.INT_TYPE, Type.DOUBLE_TYPE);
                break;
            }
            case 136: {
                this.cast(Type.LONG_TYPE, Type.INT_TYPE);
                break;
            }
            case 137: {
                this.cast(Type.LONG_TYPE, Type.FLOAT_TYPE);
                break;
            }
            case 138: {
                this.cast(Type.LONG_TYPE, Type.DOUBLE_TYPE);
                break;
            }
            case 139: {
                this.cast(Type.FLOAT_TYPE, Type.INT_TYPE);
                break;
            }
            case 140: {
                this.cast(Type.FLOAT_TYPE, Type.LONG_TYPE);
                break;
            }
            case 141: {
                this.cast(Type.FLOAT_TYPE, Type.DOUBLE_TYPE);
                break;
            }
            case 142: {
                this.cast(Type.DOUBLE_TYPE, Type.INT_TYPE);
                break;
            }
            case 143: {
                this.cast(Type.DOUBLE_TYPE, Type.LONG_TYPE);
                break;
            }
            case 144: {
                this.cast(Type.DOUBLE_TYPE, Type.FLOAT_TYPE);
                break;
            }
            case 145: {
                this.cast(Type.INT_TYPE, Type.BYTE_TYPE);
                break;
            }
            case 146: {
                this.cast(Type.INT_TYPE, Type.CHAR_TYPE);
                break;
            }
            case 147: {
                this.cast(Type.INT_TYPE, Type.SHORT_TYPE);
                break;
            }
            case 148: {
                this.lcmp();
                break;
            }
            case 149: {
                this.cmpl(Type.FLOAT_TYPE);
                break;
            }
            case 150: {
                this.cmpg(Type.FLOAT_TYPE);
                break;
            }
            case 151: {
                this.cmpl(Type.DOUBLE_TYPE);
                break;
            }
            case 152: {
                this.cmpg(Type.DOUBLE_TYPE);
                break;
            }
            case 172: {
                this.areturn(Type.INT_TYPE);
                break;
            }
            case 173: {
                this.areturn(Type.LONG_TYPE);
                break;
            }
            case 174: {
                this.areturn(Type.FLOAT_TYPE);
                break;
            }
            case 175: {
                this.areturn(Type.DOUBLE_TYPE);
                break;
            }
            case 176: {
                this.areturn(InstructionAdapter.OBJECT_TYPE);
                break;
            }
            case 177: {
                this.areturn(Type.VOID_TYPE);
                break;
            }
            case 190: {
                this.arraylength();
                break;
            }
            case 191: {
                this.athrow();
                break;
            }
            case 194: {
                this.monitorenter();
                break;
            }
            case 195: {
                this.monitorexit();
                break;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    @Override
    public void visitIntInsn(int n, int n2) {
        switch (n) {
            case 16: {
                this.iconst(n2);
                break;
            }
            case 17: {
                this.iconst(n2);
                break;
            }
            case 188: {
                switch (n2) {
                    case 4: {
                        this.newarray(Type.BOOLEAN_TYPE);
                        break;
                    }
                    case 5: {
                        this.newarray(Type.CHAR_TYPE);
                        break;
                    }
                    case 8: {
                        this.newarray(Type.BYTE_TYPE);
                        break;
                    }
                    case 9: {
                        this.newarray(Type.SHORT_TYPE);
                        break;
                    }
                    case 10: {
                        this.newarray(Type.INT_TYPE);
                        break;
                    }
                    case 6: {
                        this.newarray(Type.FLOAT_TYPE);
                        break;
                    }
                    case 11: {
                        this.newarray(Type.LONG_TYPE);
                        break;
                    }
                    case 7: {
                        this.newarray(Type.DOUBLE_TYPE);
                        break;
                    }
                    default: {
                        throw new IllegalArgumentException();
                    }
                }
                break;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    @Override
    public void visitVarInsn(int n, int n2) {
        switch (n) {
            case 21: {
                this.load(n2, Type.INT_TYPE);
                break;
            }
            case 22: {
                this.load(n2, Type.LONG_TYPE);
                break;
            }
            case 23: {
                this.load(n2, Type.FLOAT_TYPE);
                break;
            }
            case 24: {
                this.load(n2, Type.DOUBLE_TYPE);
                break;
            }
            case 25: {
                this.load(n2, InstructionAdapter.OBJECT_TYPE);
                break;
            }
            case 54: {
                this.store(n2, Type.INT_TYPE);
                break;
            }
            case 55: {
                this.store(n2, Type.LONG_TYPE);
                break;
            }
            case 56: {
                this.store(n2, Type.FLOAT_TYPE);
                break;
            }
            case 57: {
                this.store(n2, Type.DOUBLE_TYPE);
                break;
            }
            case 58: {
                this.store(n2, InstructionAdapter.OBJECT_TYPE);
                break;
            }
            case 169: {
                this.ret(n2);
                break;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    @Override
    public void visitTypeInsn(int n, String internalName) {
        Type objectType = Type.getObjectType(internalName);
        switch (n) {
            case 187: {
                this.anew(objectType);
                break;
            }
            case 189: {
                this.newarray(objectType);
                break;
            }
            case 192: {
                this.checkcast(objectType);
                break;
            }
            case 193: {
                this.instanceOf(objectType);
                break;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    @Override
    public void visitFieldInsn(int n, String s, String s2, String s3) {
        switch (n) {
            case 178: {
                this.getstatic(s, s2, s3);
                break;
            }
            case 179: {
                this.putstatic(s, s2, s3);
                break;
            }
            case 180: {
                this.getfield(s, s2, s3);
                break;
            }
            case 181: {
                this.putfield(s, s2, s3);
                break;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    @Deprecated
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, opcode == 185);
        return;
    }
    
    @Override
    public void visitMethodInsn(int opcode, String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            super.visitMethodInsn(opcode, owner, name, descriptor, isInterface);
            return;
        }
        this.doVisitMethodInsn(opcode, owner, name, descriptor, isInterface);
        return;
    }
    
    private void doVisitMethodInsn(int n, String s, String s2, String s3, boolean b) {
        switch (n) {
            case 183: {
                this.invokespecial(s, s2, s3, b);
                break;
            }
            case 182: {
                this.invokevirtual(s, s2, s3, b);
                break;
            }
            case 184: {
                this.invokestatic(s, s2, s3, b);
                break;
            }
            case 185: {
                this.invokeinterface(s, s2, s3);
                break;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    @Override
    public void visitInvokeDynamicInsn(String s, String s2, Handle handle, Object... array) {
        this.invokedynamic(s, s2, handle, array);
    }
    
    @Override
    public void visitJumpInsn(int n, Label label) {
        switch (n) {
            case 153: {
                this.ifeq(label);
                break;
            }
            case 154: {
                this.ifne(label);
                break;
            }
            case 155: {
                this.iflt(label);
                break;
            }
            case 156: {
                this.ifge(label);
                break;
            }
            case 157: {
                this.ifgt(label);
                break;
            }
            case 158: {
                this.ifle(label);
                break;
            }
            case 159: {
                this.ificmpeq(label);
                break;
            }
            case 160: {
                this.ificmpne(label);
                break;
            }
            case 161: {
                this.ificmplt(label);
                break;
            }
            case 162: {
                this.ificmpge(label);
                break;
            }
            case 163: {
                this.ificmpgt(label);
                break;
            }
            case 164: {
                this.ificmple(label);
                break;
            }
            case 165: {
                this.ifacmpeq(label);
                break;
            }
            case 166: {
                this.ifacmpne(label);
                break;
            }
            case 167: {
                this.goTo(label);
                break;
            }
            case 168: {
                this.jsr(label);
                break;
            }
            case 198: {
                this.ifnull(label);
                break;
            }
            case 199: {
                this.ifnonnull(label);
                break;
            }
            default: {
                throw new IllegalArgumentException();
            }
        }
    }
    
    @Override
    public void visitLabel(Label label) {
        this.mark(label);
    }
    
    @Override
    public void visitLdcInsn(Object o) {
        if ((o instanceof Integer)) {
            this.iconst((int)o);
        }
        else if ((o instanceof Byte)) {
            this.iconst((int)(Byte)o);
        }
        else if ((o instanceof Character)) {
            this.iconst((int)(char)o);
        }
        else if ((o instanceof Short)) {
            this.iconst((int)(Short)o);
        }
        else if ((o instanceof Boolean)) {
            this.iconst((int)(boolean)o);
        }
        else if ((o instanceof Float)) {
            this.fconst((float)o);
        }
        else if ((o instanceof Long)) {
            this.lconst((long)o);
        }
        else if ((o instanceof Double)) {
            this.dconst((double)o);
        }
        else if ((o instanceof String)) {
            this.aconst(o);
        }
        else if ((o instanceof Type)) {
            this.tconst((Type)o);
        }
        else if ((o instanceof Handle)) {
            this.hconst((Handle)o);
        }
        else {
            throw new IllegalArgumentException();
        }
    }
    
    @Override
    public void visitIincInsn(int n, int n2) {
        this.iinc(n, n2);
    }
    
    @Override
    public void visitTableSwitchInsn(int n, int n2, Label label, Label... array) {
        this.tableswitch(n, n2, label, array);
    }
    
    @Override
    public void visitLookupSwitchInsn(Label label, int[] array, Label[] array2) {
        this.lookupswitch(label, array, array2);
    }
    
    @Override
    public void visitMultiANewArrayInsn(String s, int n) {
        this.multianewarray(s, n);
    }
    
    public void nop() {
        this.mv.visitInsn(0);
    }
    
    public void aconst(Object value) {
        if (value == null) {
            this.mv.visitInsn(1);
        }
        else {
            this.mv.visitLdcInsn(value);
        }
    }
    
    public void iconst(int operand) {
        if (operand >= -1 && operand <= 5) {
            this.mv.visitInsn(3 + operand);
        }
        else if (operand >= -128 && operand <= 127) {
            this.mv.visitIntInsn(16, operand);
        }
        else if (operand >= -32768 && operand <= 32767) {
            this.mv.visitIntInsn(17, operand);
        }
        else {
            this.mv.visitLdcInsn(Integer.valueOf(operand));
        }
    }
    
    public void lconst(long l) {
        if (l == 0L || l == 1L) {
            this.mv.visitInsn(9 + (int)l);
        }
        else {
            this.mv.visitLdcInsn(Long.valueOf(l));
        }
    }
    
    public void fconst(float n) {
        int floatToIntBits = Float.floatToIntBits(n);
        if ((long)floatToIntBits == 0L || floatToIntBits == 1065353216 || floatToIntBits == 1073741824) {
            this.mv.visitInsn(11 + (int)n);
        }
        else {
            this.mv.visitLdcInsn(Float.valueOf(n));
        }
    }
    
    public void dconst(double n) {
        long doubleToLongBits = Double.doubleToLongBits(n);
        if (doubleToLongBits == 0L || doubleToLongBits == 4607182418800017408L) {
            this.mv.visitInsn(14 + (int)n);
        }
        else {
            this.mv.visitLdcInsn(Double.valueOf(n));
        }
    }
    
    public void tconst(Type value) {
        this.mv.visitLdcInsn(value);
    }
    
    public void hconst(Handle value) {
        this.mv.visitLdcInsn(value);
    }
    
    public void load(int varIndex, Type type) {
        this.mv.visitVarInsn(type.getOpcode(21), varIndex);
    }
    
    public void aload(Type type) {
        this.mv.visitInsn(type.getOpcode(46));
    }
    
    public void store(int varIndex, Type type) {
        this.mv.visitVarInsn(type.getOpcode(54), varIndex);
    }
    
    public void astore(Type type) {
        this.mv.visitInsn(type.getOpcode(79));
    }
    
    public void pop() {
        this.mv.visitInsn(87);
    }
    
    public void pop2() {
        this.mv.visitInsn(88);
    }
    
    public void dup() {
        this.mv.visitInsn(89);
    }
    
    public void dup2() {
        this.mv.visitInsn(92);
    }
    
    public void dupX1() {
        this.mv.visitInsn(90);
    }
    
    public void dupX2() {
        this.mv.visitInsn(91);
    }
    
    public void dup2X1() {
        this.mv.visitInsn(93);
    }
    
    public void dup2X2() {
        this.mv.visitInsn(94);
    }
    
    public void swap() {
        this.mv.visitInsn(95);
    }
    
    public void add(Type type) {
        this.mv.visitInsn(type.getOpcode(96));
    }
    
    public void sub(Type type) {
        this.mv.visitInsn(type.getOpcode(100));
    }
    
    public void mul(Type type) {
        this.mv.visitInsn(type.getOpcode(104));
    }
    
    public void div(Type type) {
        this.mv.visitInsn(type.getOpcode(108));
    }
    
    public void rem(Type type) {
        this.mv.visitInsn(type.getOpcode(112));
    }
    
    public void neg(Type type) {
        this.mv.visitInsn(type.getOpcode(116));
    }
    
    public void shl(Type type) {
        this.mv.visitInsn(type.getOpcode(120));
    }
    
    public void shr(Type type) {
        this.mv.visitInsn(type.getOpcode(122));
    }
    
    public void ushr(Type type) {
        this.mv.visitInsn(type.getOpcode(124));
    }
    
    public void and(Type type) {
        this.mv.visitInsn(type.getOpcode(126));
    }
    
    public void or(Type type) {
        this.mv.visitInsn(type.getOpcode(128));
    }
    
    public void xor(Type type) {
        this.mv.visitInsn(type.getOpcode(130));
    }
    
    public void iinc(int varIndex, int increment) {
        this.mv.visitIincInsn(varIndex, increment);
    }
    
    public void cast(Type type, Type type2) {
        if (type != type2) {
            if (type == Type.DOUBLE_TYPE) {
                if (type2 == Type.FLOAT_TYPE) {
                    this.mv.visitInsn(144);
                }
                else if (type2 == Type.LONG_TYPE) {
                    this.mv.visitInsn(143);
                }
                else {
                    this.mv.visitInsn(142);
                    this.cast(Type.INT_TYPE, type2);
                }
            }
            else if (type == Type.FLOAT_TYPE) {
                if (type2 == Type.DOUBLE_TYPE) {
                    this.mv.visitInsn(141);
                }
                else if (type2 == Type.LONG_TYPE) {
                    this.mv.visitInsn(140);
                }
                else {
                    this.mv.visitInsn(139);
                    this.cast(Type.INT_TYPE, type2);
                }
            }
            else if (type == Type.LONG_TYPE) {
                if (type2 == Type.DOUBLE_TYPE) {
                    this.mv.visitInsn(138);
                }
                else if (type2 == Type.FLOAT_TYPE) {
                    this.mv.visitInsn(137);
                }
                else {
                    this.mv.visitInsn(136);
                    this.cast(Type.INT_TYPE, type2);
                }
            }
            else if (type2 == Type.BYTE_TYPE) {
                this.mv.visitInsn(145);
            }
            else if (type2 == Type.CHAR_TYPE) {
                this.mv.visitInsn(146);
            }
            else if (type2 == Type.DOUBLE_TYPE) {
                this.mv.visitInsn(135);
            }
            else if (type2 == Type.FLOAT_TYPE) {
                this.mv.visitInsn(134);
            }
            else if (type2 == Type.LONG_TYPE) {
                this.mv.visitInsn(133);
            }
            else if (type2 == Type.SHORT_TYPE) {
                this.mv.visitInsn(147);
            }
        }
    }
    
    public void lcmp() {
        this.mv.visitInsn(148);
    }
    
    public void cmpl(Type type) {
        this.mv.visitInsn((type == Type.FLOAT_TYPE) ? 149 : 151);
    }
    
    public void cmpg(Type type) {
        this.mv.visitInsn((type == Type.FLOAT_TYPE) ? 150 : 152);
    }
    
    public void ifeq(Label label) {
        this.mv.visitJumpInsn(153, label);
    }
    
    public void ifne(Label label) {
        this.mv.visitJumpInsn(154, label);
    }
    
    public void iflt(Label label) {
        this.mv.visitJumpInsn(155, label);
    }
    
    public void ifge(Label label) {
        this.mv.visitJumpInsn(156, label);
    }
    
    public void ifgt(Label label) {
        this.mv.visitJumpInsn(157, label);
    }
    
    public void ifle(Label label) {
        this.mv.visitJumpInsn(158, label);
    }
    
    public void ificmpeq(Label label) {
        this.mv.visitJumpInsn(159, label);
    }
    
    public void ificmpne(Label label) {
        this.mv.visitJumpInsn(160, label);
    }
    
    public void ificmplt(Label label) {
        this.mv.visitJumpInsn(161, label);
    }
    
    public void ificmpge(Label label) {
        this.mv.visitJumpInsn(162, label);
    }
    
    public void ificmpgt(Label label) {
        this.mv.visitJumpInsn(163, label);
    }
    
    public void ificmple(Label label) {
        this.mv.visitJumpInsn(164, label);
    }
    
    public void ifacmpeq(Label label) {
        this.mv.visitJumpInsn(165, label);
    }
    
    public void ifacmpne(Label label) {
        this.mv.visitJumpInsn(166, label);
    }
    
    public void goTo(Label label) {
        this.mv.visitJumpInsn(167, label);
    }
    
    public void jsr(Label label) {
        this.mv.visitJumpInsn(168, label);
    }
    
    public void ret(int varIndex) {
        this.mv.visitVarInsn(169, varIndex);
    }
    
    public void tableswitch(int min, int max, Label dflt, Label... labels) {
        this.mv.visitTableSwitchInsn(min, max, dflt, labels);
    }
    
    public void lookupswitch(Label dflt, int[] keys, Label[] labels) {
        this.mv.visitLookupSwitchInsn(dflt, keys, labels);
    }
    
    public void areturn(Type type) {
        this.mv.visitInsn(type.getOpcode(172));
    }
    
    public void getstatic(String owner, String name, String descriptor) {
        this.mv.visitFieldInsn(178, owner, name, descriptor);
    }
    
    public void putstatic(String owner, String name, String descriptor) {
        this.mv.visitFieldInsn(179, owner, name, descriptor);
    }
    
    public void getfield(String owner, String name, String descriptor) {
        this.mv.visitFieldInsn(180, owner, name, descriptor);
    }
    
    public void putfield(String owner, String name, String descriptor) {
        this.mv.visitFieldInsn(181, owner, name, descriptor);
    }
    
    @Deprecated
    public void invokevirtual(String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            this.invokevirtual(owner, name, descriptor, false);
            return;
        }
        this.mv.visitMethodInsn(182, owner, name, descriptor);
        return;
    }
    
    public void invokevirtual(String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            if (isInterface) {
                throw new IllegalArgumentException("INVOKEVIRTUAL on interfaces require ASM 5");
            }
            this.invokevirtual(owner, name, descriptor);
            return;
        }
        else {
            this.mv.visitMethodInsn(182, owner, name, descriptor, isInterface);
            return;
        }
    }
    
    @Deprecated
    public void invokespecial(String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            this.invokespecial(owner, name, descriptor, false);
            return;
        }
        this.mv.visitMethodInsn(183, owner, name, descriptor, false);
        return;
    }
    
    public void invokespecial(String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            if (isInterface) {
                throw new IllegalArgumentException("INVOKESPECIAL on interfaces require ASM 5");
            }
            this.invokespecial(owner, name, descriptor);
            return;
        }
        else {
            this.mv.visitMethodInsn(183, owner, name, descriptor, isInterface);
            return;
        }
    }
    
    @Deprecated
    public void invokestatic(String owner, String name, String descriptor) {
        if (this.api >= 327680) {
            this.invokestatic(owner, name, descriptor, false);
            return;
        }
        this.mv.visitMethodInsn(184, owner, name, descriptor, false);
        return;
    }
    
    public void invokestatic(String owner, String name, String descriptor, boolean isInterface) {
        if (this.api < 327680) {
            if (isInterface) {
                throw new IllegalArgumentException("INVOKESTATIC on interfaces require ASM 5");
            }
            this.invokestatic(owner, name, descriptor);
            return;
        }
        else {
            this.mv.visitMethodInsn(184, owner, name, descriptor, isInterface);
            return;
        }
    }
    
    public void invokeinterface(String owner, String name, String descriptor) {
        this.mv.visitMethodInsn(185, owner, name, descriptor, true);
    }
    
    public void invokedynamic(String name, String descriptor, Handle bootstrapMethodHandle, Object[] bootstrapMethodArguments) {
        this.mv.visitInvokeDynamicInsn(name, descriptor, bootstrapMethodHandle, bootstrapMethodArguments);
    }
    
    public void anew(Type type) {
        this.mv.visitTypeInsn(187, type.getInternalName());
    }
    
    public void newarray(Type type) {
        int operand;
        switch (type.getSort()) {
            case 1: {
                operand = 4;
                break;
            }
            case 2: {
                operand = 5;
                break;
            }
            case 3: {
                operand = 8;
                break;
            }
            case 4: {
                operand = 9;
                break;
            }
            case 5: {
                operand = 10;
                break;
            }
            case 6: {
                operand = 6;
                break;
            }
            case 7: {
                operand = 11;
                break;
            }
            case 8: {
                operand = 7;
                break;
            }
            default: {
                this.mv.visitTypeInsn(189, type.getInternalName());
                return;
            }
        }
        this.mv.visitIntInsn(188, operand);
    }
    
    public void arraylength() {
        this.mv.visitInsn(190);
    }
    
    public void athrow() {
        this.mv.visitInsn(191);
    }
    
    public void checkcast(Type type) {
        this.mv.visitTypeInsn(192, type.getInternalName());
    }
    
    public void instanceOf(Type type) {
        this.mv.visitTypeInsn(193, type.getInternalName());
    }
    
    public void monitorenter() {
        this.mv.visitInsn(194);
    }
    
    public void monitorexit() {
        this.mv.visitInsn(195);
    }
    
    public void multianewarray(String descriptor, int numDimensions) {
        this.mv.visitMultiANewArrayInsn(descriptor, numDimensions);
    }
    
    public void ifnull(Label label) {
        this.mv.visitJumpInsn(198, label);
    }
    
    public void ifnonnull(Label label) {
        this.mv.visitJumpInsn(199, label);
    }
    
    public void mark(Label label) {
        this.mv.visitLabel(label);
    }
    
    static {
        OBJECT_TYPE = Type.getType("Ljava/lang/Object;");
    }
}
